#ifndef REMOTE_FILE_ITEM_H
#define REMOTE_FILE_ITEM_H

#include "queue/PortableQueue.h"
#include "message/PortableMessage.h"

#include "CLVListItem.h"
#include "ShareConstants.h"

class RemoteUserItem;
class ColumnListView;

// Represents information about a file that is available for download.
// We use objects of this class as keys in our search results hash table.
class RemoteFileItem : public CLVListItem
{
public:
   RemoteFileItem(RemoteUserItem * owner, const char * fileName, PortableMessageRef attrs);
   ~RemoteFileItem();

   const char * GetFileName() const {return _fileName.Cstr();}

   virtual void DrawItemColumn(BView * owner, BRect rect, int32 colIdx, bool complete);

   const PortableMessage & GetAttributes() const {return *_attributes.GetItemPointer();}

   void Update(BView * view, const BFont * font);

   int Compare(const RemoteFileItem * item2, int32 key) const;

   RemoteUserItem * GetOwner() const {return _owner;}

private:
   RemoteUserItem * _owner;
   PortableString _fileName;
   PortableMessageRef _attributes;      
   float _textOffset;
};

#endif
