#include "RemoteUserItem.h"
#include "RemoteFileItem.h"
#include "ShareStrings.h"
#include "ShareWindow.h"

RemoteUserItem ::
RemoteUserItem(ShareWindow * owner, const char * sessionID)
   : _owner(owner), _sessionID(sessionID), _handle(str(STR_ANONYMOUS)), _port(0), _firewalled(false), _isBot(false), _files(CStringHashFunc), _bandwidthLabel(str(STR_UNKNOWN)), _bandwidth(0)
{
   _files.SetCompareFunction(CStringCompareFunc);
 
   PortableString text = GetUserString();
   text += str(STR_IS_NOW_CONNECTED); 
   _owner->LogMessage(LOG_USER_EVENT_MESSAGE, text.Cstr());

   SetColumnContent(0, _handle.Cstr(), false, false);
   SetColumnContent(1, _sessionID.Cstr(), false, true);
   SetNumSharedFiles(-1);
}

RemoteUserItem ::
~RemoteUserItem()
{
   PortableString text = GetUserString();
   text += str(STR_HAS_DISCONNECTED);
   _owner->LogMessage(LOG_USER_EVENT_MESSAGE, text.Cstr());
   ClearFiles();
}

PortableString
RemoteUserItem ::
GetUserString() const
{
   PortableString text(str(STR_USER_NUMBER));
   text += _sessionID;

   if (!_handle.Equals(str(STR_ANONYMOUS)))
   {
      text += str(STR_AKA);
      text += _handle.Cstr();
      text += ")";
   }
   return text;
}

void
RemoteUserItem ::
SetHandle(const char * handle)
{
   PortableString text = GetUserString();
   text += str(STR_IS_NOW_KNOWN_AS);
   text += handle;
   _owner->LogMessage(LOG_USER_EVENT_MESSAGE, text.Cstr());

   _handle = handle;
   SetColumnContent(0, _handle.Cstr(), false, false);
   _owner->RefreshUserItem(this);

   // Make sure all our files refresh too, in case the "owner name" column is visible
   RemoteFileItem * next;
   PortableHashtableIterator<const char *, RemoteFileItem *> iter = _files.GetIterator();
   while(iter.GetNextValue(next)) _owner->RefreshFileItem(next);
}

void
RemoteUserItem ::
SetNumSharedFiles(int32 bw)
{
   _numSharedFiles = bw;
  
   char temp[100] = "?";
   if (_numSharedFiles >= 0) 
   {
      if ((_firewalled)&&(_owner->GetFirewalled())) sprintf(temp, "(%li)", _numSharedFiles);
                                               else sprintf(temp, "%li", _numSharedFiles);
   }
   SetColumnContent(2, temp, false, true);
   _owner->RefreshUserItem(this);
}

void
RemoteUserItem ::
SetBandwidth(const char * bandwidthLabel, uint32 bps)
{
   _bandwidthLabel = bandwidthLabel;
   _bandwidth = bps;

   // Make sure all our files refresh too, in case the "owner bps" column is visible
   RemoteFileItem * next;
   PortableHashtableIterator<const char *, RemoteFileItem *> iter = _files.GetIterator();
   while(iter.GetNextValue(next)) _owner->RefreshFileItem(next);

   SetColumnContent(3, _bandwidthLabel.Cstr(), false, false);
   _owner->RefreshUserItem(this);
}

void
RemoteUserItem ::
ClearFiles()
{
   PortableHashtableIterator<const char *, RemoteFileItem *> iter = _files.GetIterator();
   RemoteFileItem * next;
   while(iter.GetNextValue(next)) 
   {
      _owner->RemoveFileItem(next);
      delete next;
   }
   _files.Clear();
}

void
RemoteUserItem ::
PutFile(const char * fileName, PortableMessageRef fileAttrs)
{
   RemoveFile(fileName);
   RemoteFileItem * item = new RemoteFileItem(this, fileName, fileAttrs);
   _files.Put(item->GetFileName(), item);
   _owner->AddFileItem(item);
}


void
RemoteUserItem ::
RemoveFile(const char * fileName)
{
   RemoteFileItem * item;
   if (_files.Remove(fileName, item))
   {
      _owner->RemoveFileItem(item);
      delete item;
   }
}


void
RemoteUserItem ::
SetFirewalled(bool fw)
{
   if (fw != _firewalled)
   {
      _firewalled = fw;
      SetNumSharedFiles(GetNumSharedFiles());  // force update
   }
}

void 
RemoteUserItem :: 
DrawItemColumn(BView* owner, BRect rect, int32 col, bool complete)
{
   if ((_isBot)&&(col == 0))
   {
      BFont f;
      owner->GetFont(&f);
      font_family family; 
      font_style style;
      f.GetFamilyAndStyle(&family, &style);
      f.SetFamilyAndStyle(family, "Italic");
      owner->SetFont(&f);
      CLVEasyItem::DrawItemColumn(owner, rect, col, complete);
      f.SetFamilyAndStyle(family, style);
      owner->SetFont(&f);
   }
   else CLVEasyItem::DrawItemColumn(owner, rect, col, complete);
}

