#include <app/Application.h>
#include <storage/AppFileInfo.h>
#include <storage/Path.h>
#include <storage/File.h>
#include <storage/FindDirectory.h>

#include "ShareWindow.h"

#define BESHARE_SETTINGS_FILE_NAME "beshare_settings"   

class ShareApplication : public BApplication
{
public:
   ShareApplication(const char * optConnectTo) : BApplication(BESHARE_MIME_TYPE), _window(NULL), _optConnectTo(optConnectTo)
   {
      // Get default settings path
      BPath sPath;
      if (find_directory(B_USER_SETTINGS_DIRECTORY, &sPath) == B_NO_ERROR)
      {
         sPath.Append(BESHARE_SETTINGS_FILE_NAME);
         _settingsFileEntry.SetTo(sPath.Path());
      }
   }

   ~ShareApplication()
   {
      // Attempt to save settings...
      BFile settingsFile(&_settingsFileEntry, B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
      if (settingsFile.InitCheck() == B_NO_ERROR) 
      {
         (void) _settingsMsg.Flatten(&settingsFile);
         BAppFileInfo afi(&settingsFile);
         afi.SetType(BESHARE_MIME_TYPE);
      }
   }

   virtual void RefsReceived(BMessage * msg)
   {
      entry_ref ref; 
      if (msg->FindRef("refs", &ref) == B_NO_ERROR) _settingsFileEntry.SetTo(&ref);
   }
  
   virtual void ReadyToRun()
   {
      // Attempt to load settings...
      BFile settingsFile(&_settingsFileEntry, B_READ_ONLY);
      if (settingsFile.InitCheck() == B_NO_ERROR) (void) _settingsMsg.Unflatten(&settingsFile);
      _window = new ShareWindow(_settingsMsg, _optConnectTo);
      _window->Show();
   }

   virtual void DispatchMessage(BMessage * msg, BHandler * handler)
   {
      if (msg->what == B_QUIT_REQUESTED)
      {
         if (msg->IsSourceRemote())
         {
            // Tell our window not to put up an 'are you sure' requester
            if ((_window)&&(_window->Lock()))
            {
               _window->SetEnableQuitRequester(false);
               _window->Unlock();
            }
         }
      }
      BApplication::DispatchMessage(msg, handler);
   }

private:
   BMessage _settingsMsg;
   ShareWindow * _window;
   BEntry _settingsFileEntry;
   const char * _optConnectTo;
};

int main(int argc, char ** argv)
{
   (ShareApplication((argc>1)?argv[1]:NULL)).Run();
   return 0;
}
